@@ -128,9 +128,6 @@ oobs_user_class_init (OobsUserClass *class)
   oobs_class->update = oobs_user_update;
   oobs_class->get_update_message = oobs_user_get_update_message;
 
-  /* override the singleton check */
-  oobs_class->singleton = FALSE;
-
   g_object_class_install_property (object_class,
 				   PROP_USERNAME,
 				   g_param_spec_string ("name",
@@ -594,7 +591,7 @@ create_dbus_struct_from_user (OobsUser        *user,
   utils_append_string (iter, shell);
   utils_append_int (iter, passwd_flags);
   utils_append_boolean (iter, enc_home);
-  utils_append_boolean (iter, home_flags);
+  utils_append_int (iter, home_flags);
   utils_append_string (iter, locale);
   /* TODO: use location and face when the backends support it */
   utils_append_string (iter, "");
@@ -630,7 +627,8 @@ oobs_user_commit (OobsObject *object)
 
   /* Erase password field as soon as possible */
   priv = OOBS_USER_GET_PRIVATE (OOBS_USER (object));
-  memset (priv->password, 0, strlen (priv->password));
+  if (priv->password)
+    memset (priv->password, 0, strlen (priv->password));
 }
 
 /*
