@@ -15,7 +15,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  *
- * Authors: Carlos Garnacho Parro  <carlosg@gnome.org>
+ * Authors: Carlos Garnacho Parro  <carlosg@gnome.org>,
+ *          Milan Bouchet-Valat <nalimilan@club.fr>.
  */
 
 #include <dbus/dbus.h>
@@ -28,6 +29,7 @@
 #include "oobs-list-private.h"
 #include "oobs-usersconfig.h"
 #include "oobs-user.h"
+#include "oobs-user-private.h"
 #include "oobs-defines.h"
 #include "oobs-groupsconfig.h"
 #include "oobs-group.h"
@@ -40,7 +42,7 @@
  * @see_also: #OobsUser
  **/
 
-#define USERS_CONFIG_REMOTE_OBJECT "UsersConfig"
+#define USERS_CONFIG_REMOTE_OBJECT "UsersConfig2"
 #define OOBS_USERS_CONFIG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), OOBS_TYPE_USERS_CONFIG, OobsUsersConfigPrivate))
 
 typedef struct _OobsUsersConfigPrivate OobsUsersConfigPrivate;
@@ -51,14 +53,15 @@ struct _OobsUsersConfigPrivate
 
   GList    *shells;
 
-  gboolean  use_md5;
   uid_t     minimum_uid;
   uid_t     maximum_uid;
   gchar    *default_shell;
   gchar    *default_home;
 
   GHashTable *groups;
-  gint        default_gid;
+  gid_t       default_gid;
+
+  gboolean  encrypted_home;
 
   OobsGroup *default_group;
 };
@@ -86,12 +89,12 @@ static void oobs_users_config_commit     (OobsObject   *object);
 enum
 {
   PROP_0,
-  PROP_USE_MD5,
   PROP_MINIMUM_UID,
   PROP_MAXIMUM_UID,
   PROP_DEFAULT_SHELL,
   PROP_DEFAULT_HOME,
-  PROP_DEFAULT_GROUP
+  PROP_DEFAULT_GROUP,
+  PROP_ENCRYPTED_HOME,
 };
 
 G_DEFINE_TYPE (OobsUsersConfig, oobs_users_config, OOBS_TYPE_OBJECT);
@@ -111,27 +114,20 @@ oobs_users_config_class_init (OobsUsersConfigClass *class)
   oobs_object_class->commit  = oobs_users_config_commit;
   oobs_object_class->update  = oobs_users_config_update;
 
-  g_object_class_install_property (object_class,
-				   PROP_USE_MD5,
-				   g_param_spec_boolean ("use-md5",
-							 "Use MD5",
-							 "Whether users' passwords are stored in MD5",
-							 FALSE,
-							 G_PARAM_READABLE));
   g_object_class_install_property (object_class,
 				   PROP_MINIMUM_UID,
-				   g_param_spec_int ("minimum-uid",
-						     "Minimum UID",
-						     "Minimum UID for non-system users",
-						     0, OOBS_MAX_UID, OOBS_MAX_UID,
-						     G_PARAM_READWRITE));
+				   g_param_spec_uint ("minimum-uid",
+						      "Minimum UID",
+				                      "Minimum UID for non-system users",
+						      0, OOBS_MAX_UID, OOBS_MAX_UID,
+						      G_PARAM_READWRITE));
   g_object_class_install_property (object_class,
 				   PROP_MAXIMUM_UID,
-				   g_param_spec_int ("maximum-uid",
-						     "Maximum UID",
-						     "Maximum UID for non-system users",
-						     0, OOBS_MAX_UID, OOBS_MAX_UID,
-						     G_PARAM_READWRITE));
+				   g_param_spec_uint ("maximum-uid",
+				                      "Maximum UID",
+				                      "Maximum UID for non-system users",
+				                      0, OOBS_MAX_UID, OOBS_MAX_UID,
+				                      G_PARAM_READWRITE));
   g_object_class_install_property (object_class,
 				   PROP_DEFAULT_SHELL,
 				   g_param_spec_string ("default-shell",
@@ -153,6 +149,14 @@ oobs_users_config_class_init (OobsUsersConfigClass *class)
 							"Default group for new users",
 							OOBS_TYPE_GROUP,
 							G_PARAM_READABLE));
+  g_object_class_install_property (object_class,
+				   PROP_ENCRYPTED_HOME,
+				   g_param_spec_boolean ("encrypted-home",
+				                         "Encrypted home support",
+				                         "Whether encrypted home dirs are supported",
+				                         FALSE,
+				                         G_PARAM_READABLE));
+
   g_type_class_add_private (object_class,
 			    sizeof (OobsUsersConfigPrivate));
 }
@@ -168,7 +172,7 @@ oobs_users_config_init (OobsUsersConfig *config)
   priv->users_list = _oobs_list_new (OOBS_TYPE_USER);
   config->_priv = priv;
 
-  priv->groups = g_hash_table_new_full (NULL, NULL, g_object_unref, NULL);
+  priv->groups = g_hash_table_new (NULL, NULL);
 }
 
 static void
@@ -237,10 +241,10 @@ oobs_users_config_set_property (GObject      *object,
   switch (prop_id)
     {
     case PROP_MINIMUM_UID:
-      priv->minimum_uid = g_value_get_int (value);
+      priv->minimum_uid = g_value_get_uint (value);
       break;
     case PROP_MAXIMUM_UID:
-      priv->maximum_uid = g_value_get_int (value);
+      priv->maximum_uid = g_value_get_uint (value);
       break;
     case PROP_DEFAULT_SHELL:
       g_free (priv->default_shell);
@@ -267,14 +271,11 @@ oobs_users_config_get_property (GObject      *object,
 
   switch (prop_id)
     {
-    case PROP_USE_MD5:
-      g_value_set_boolean (value, priv->use_md5);
-      break;
     case PROP_MINIMUM_UID:
-      g_value_set_int (value, priv->minimum_uid);
+      g_value_set_uint (value, priv->minimum_uid);
       break;
     case PROP_MAXIMUM_UID:
-      g_value_set_int (value, priv->maximum_uid);
+      g_value_set_uint (value, priv->maximum_uid);
       break;
     case PROP_DEFAULT_SHELL:
       g_value_set_string (value, priv->default_shell);
@@ -282,167 +283,20 @@ oobs_users_config_get_property (GObject      *object,
     case PROP_DEFAULT_HOME:
       g_value_set_string (value, priv->default_home);
       break;
+    case PROP_ENCRYPTED_HOME:
+      g_value_set_boolean (value, priv->encrypted_home);
     }
 }
 
-static OobsUser*
-create_user_from_dbus_reply (OobsObject      *object,
-			     DBusMessage     *reply,
-			     DBusMessageIter  struct_iter)
-{
-  OobsUsersConfigPrivate *priv;
-  OobsUser *user;
-  DBusMessageIter iter, gecos_iter;
-  int    uid, gid;
-  const gchar *login, *passwd, *home, *shell;
-  const gchar *name, *room_number, *work_phone, *home_phone, *other_data;
-
-  priv = OOBS_USERS_CONFIG (object)->_priv;
-  dbus_message_iter_recurse (&struct_iter, &iter);
-
-  login = utils_get_string (&iter);
-  passwd = utils_get_string (&iter);
-  uid = utils_get_int (&iter);
-  gid = utils_get_int (&iter);
-
-  /* GECOS fields */
-  dbus_message_iter_recurse (&iter, &gecos_iter);
-
-  name = utils_get_string (&gecos_iter);
-  room_number = utils_get_string (&gecos_iter);
-  work_phone = utils_get_string (&gecos_iter);
-  home_phone = utils_get_string (&gecos_iter);
-  other_data = utils_get_string (&gecos_iter);
-  /* end of GECOS fields */
-
-  dbus_message_iter_next (&iter);
-
-  home = utils_get_string (&iter);
-  shell = utils_get_string (&iter);
-
-  user = g_object_new (OOBS_TYPE_USER,
-		       "name", login,
-		       "crypted-password", passwd,
-		       "uid", uid,
-		       "home-directory", home,
-		       "shell", shell,
-		       "full-name", name,
-		       "room-number", room_number,
-		       "work-phone", work_phone,
-		       "home-phone", home_phone,
-		       "other-data", other_data,
-		       NULL);
-
-  /* keep the GID in a hashtable, this will be needed
-   * each time the groups configuration changes
-   */
-  g_hash_table_insert (priv->groups,
-		       g_object_ref (user),
-		       (gpointer) gid);
-  return user;
-}
-
-static gboolean
-create_dbus_struct_from_user (OobsUser        *user,
-			      DBusMessage     *message,
-			      DBusMessageIter *array_iter)
-{
-  OobsGroup *group;
-  gint uid, gid;
-  gchar *login, *password, *shell, *homedir;
-  gchar *name, *room_number, *work_phone, *home_phone, *other_data;
-  DBusMessageIter struct_iter, data_iter;
-
-  g_object_get (user,
-		"name", &login,
-		"crypted-password", &password,
-		"uid", &uid,
-		"home-directory", &homedir,
-		"shell", &shell,
-		"full-name", &name,
-		"room-number", &room_number,
-		"work-phone", &work_phone,
-		"home-phone", &home_phone,
-		"other-data", &other_data,
-		NULL);
-
-  /* Login is the only required field,
-   * since home dir, password and shell are allowed to be empty (see man 5 passwd) */
-  g_return_val_if_fail (login, FALSE);
-
-  group = oobs_user_get_main_group (user);
-  gid = oobs_group_get_gid (group);
-
-  dbus_message_iter_open_container (array_iter, DBUS_TYPE_STRUCT, NULL, &struct_iter);
-
-  utils_append_string (&struct_iter, login);
-  utils_append_string (&struct_iter, password);
-  utils_append_int (&struct_iter, uid);
-  utils_append_int (&struct_iter, gid);
-
-  dbus_message_iter_open_container (&struct_iter, DBUS_TYPE_ARRAY, DBUS_TYPE_STRING_AS_STRING, &data_iter);
-
-  /* GECOS fields */
-  utils_append_string (&data_iter, name);
-  utils_append_string (&data_iter, room_number);
-  utils_append_string (&data_iter, work_phone);
-  utils_append_string (&data_iter, home_phone);
-  utils_append_string (&data_iter, other_data);
-
-  dbus_message_iter_close_container (&struct_iter, &data_iter);
-
-  utils_append_string (&struct_iter, homedir);
-  utils_append_string (&struct_iter, shell);
-  dbus_message_iter_close_container (array_iter, &struct_iter);
-
-  g_free (login);
-  g_free (password);
-  g_free (shell);
-  g_free (homedir);
-  g_free (name);
-  g_free (room_number);
-  g_free (work_phone);
-  g_free (home_phone);
-  g_free (other_data);
-
-  return TRUE;
-}
-
-static OobsGroup*
-get_group_with_gid (OobsGroupsConfig *config,
-		    gint              gid)
-{
-  OobsList *groups_list;
-  OobsListIter iter;
-  OobsGroup *group;
-  gboolean valid;
-
-  groups_list = oobs_groups_config_get_groups (config);
-  valid = oobs_list_get_iter_first (groups_list, &iter);
-
-  while (valid)
-    {
-      group = OOBS_GROUP (oobs_list_get (groups_list, &iter));
-
-      if (oobs_group_get_gid (group) == gid)
-	return group;
-
-      g_object_unref (group);
-      valid = oobs_list_iter_next (groups_list, &iter);
-    }
-
-  return NULL;
-}
-
 static void
 query_groups_foreach (OobsUser *user,
-		      gint      gid,
+		      gid_t     gid,
 		      gpointer  data)
 {
   OobsGroupsConfig *groups_config = OOBS_GROUPS_CONFIG (data);
   OobsGroup *group;
 
-  group = get_group_with_gid (groups_config, gid);
+  group = oobs_groups_config_get_from_gid (groups_config, gid);
   oobs_user_set_main_group (user, group);
 
   if (group)
@@ -462,7 +316,7 @@ oobs_users_config_groups_updated (OobsUsersConfig  *users,
   /* get the default group */
   if (priv->default_gid > 0)
     {
-      group = get_group_with_gid (groups, priv->default_gid);
+      group = oobs_groups_config_get_from_gid (groups, priv->default_gid);
       priv->default_group = group;
 
       if (group)
@@ -479,6 +333,7 @@ oobs_users_config_update (OobsObject *object)
   DBusMessageIter  iter, elem_iter;
   OobsListIter     list_iter;
   GObject         *user;
+  gid_t            gid;
 
   priv  = OOBS_USERS_CONFIG (object)->_priv;
   reply = _oobs_object_get_dbus_message (object);
@@ -491,11 +346,19 @@ oobs_users_config_update (OobsObject *object)
 
   while (dbus_message_iter_get_arg_type (&elem_iter) == DBUS_TYPE_STRUCT)
     {
-      user = G_OBJECT (create_user_from_dbus_reply (object, reply, elem_iter));
+      user = G_OBJECT (_oobs_user_create_from_dbus_reply (NULL, &gid, reply, elem_iter));
 
       oobs_list_append (priv->users_list, &list_iter);
       oobs_list_set    (priv->users_list, &list_iter, G_OBJECT (user));
-      g_object_unref   (user);
+
+      g_object_unref (user);
+
+      /* keep the group name in a hashtable, this will be needed
+       * each time the groups configuration changes
+       */
+      g_hash_table_insert (priv->groups,
+                           user,
+                           (gpointer) gid);
 
       dbus_message_iter_next (&elem_iter);
     }
@@ -503,13 +366,13 @@ oobs_users_config_update (OobsObject *object)
   dbus_message_iter_next (&iter);
   priv->shells = utils_get_string_list_from_dbus_reply (reply, &iter);
 
-  priv->use_md5 = utils_get_int (&iter);
-  priv->minimum_uid = utils_get_int (&iter);
-  priv->maximum_uid = utils_get_int (&iter);
+  priv->minimum_uid = utils_get_uint (&iter);
+  priv->maximum_uid = utils_get_uint (&iter);
 
   priv->default_home = g_strdup (utils_get_string (&iter));
   priv->default_shell = g_strdup (utils_get_string (&iter));
-  priv->default_gid = utils_get_int (&iter);
+  priv->default_gid = utils_get_uint (&iter);
+  priv->encrypted_home = utils_get_boolean (&iter);
 
   groups_config = oobs_groups_config_get ();
 
@@ -526,63 +389,24 @@ oobs_users_config_commit (OobsObject *object)
 {
   OobsUsersConfigPrivate *priv;
   DBusMessage *message;
-  DBusMessageIter iter, array_iter;
-  OobsListIter list_iter;
-  GObject *user;
-  gboolean valid, correct;
-  gint default_gid;
+  DBusMessageIter iter;
+  guint32 default_gid;
 
   priv = OOBS_USERS_CONFIG (object)->_priv;
   message = _oobs_object_get_dbus_message (object);
 
   dbus_message_iter_init_append (message, &iter);
-  dbus_message_iter_open_container (&iter,
-				    DBUS_TYPE_ARRAY,
-				    DBUS_STRUCT_BEGIN_CHAR_AS_STRING
-				    DBUS_TYPE_STRING_AS_STRING
-				    DBUS_TYPE_STRING_AS_STRING
-				    DBUS_TYPE_INT32_AS_STRING
-				    DBUS_TYPE_INT32_AS_STRING
-				    DBUS_TYPE_ARRAY_AS_STRING
-				    DBUS_TYPE_STRING_AS_STRING
-				    DBUS_TYPE_STRING_AS_STRING
-				    DBUS_TYPE_STRING_AS_STRING
-				    DBUS_STRUCT_END_CHAR_AS_STRING
-				    DBUS_TYPE_INT32_AS_STRING
-				    DBUS_TYPE_INT32_AS_STRING
-				    DBUS_TYPE_INT32_AS_STRING
-				    DBUS_TYPE_STRING_AS_STRING
-				    DBUS_TYPE_STRING_AS_STRING,
-				    &array_iter);
-  valid = oobs_list_get_iter_first (priv->users_list, &list_iter);
-  correct = TRUE;
-
-  while (valid && correct)
-    {
-      user = oobs_list_get (priv->users_list, &list_iter);
-      correct = create_dbus_struct_from_user (OOBS_USER (user), message, &array_iter);
-
-      g_object_unref (user);
-      valid = oobs_list_iter_next (priv->users_list, &list_iter);
-    }
-
-  dbus_message_iter_close_container (&iter, &array_iter);
 
   utils_create_dbus_array_from_string_list (priv->shells, message, &iter);
-  utils_append_int (&iter, priv->use_md5);
-  utils_append_int (&iter, priv->minimum_uid);
-  utils_append_int (&iter, priv->maximum_uid);
+  utils_append_uint (&iter, priv->minimum_uid);
+  utils_append_uint (&iter, priv->maximum_uid);
   utils_append_string (&iter, priv->default_home);
   utils_append_string (&iter, priv->default_shell);
 
-  default_gid = (priv->default_group) ? oobs_group_get_gid (priv->default_group) : -1;
-  utils_append_int (&iter, default_gid);
+  default_gid = (priv->default_group) ? oobs_group_get_gid (priv->default_group) : G_MAXUINT32;
+  utils_append_uint (&iter, default_gid);
 
-  if (!correct)
-    {
-      /* malformed data, unset the message */
-      _oobs_object_set_dbus_message (object, NULL);
-    }
+  utils_append_boolean (&iter, priv->encrypted_home);
 }
 
 /**
@@ -596,9 +420,14 @@ oobs_users_config_commit (OobsObject *object)
 OobsObject*
 oobs_users_config_get (void)
 {
-  return g_object_new (OOBS_TYPE_USERS_CONFIG,
-		       "remote-object", USERS_CONFIG_REMOTE_OBJECT,
-		       NULL);
+  static OobsObject *the_object = NULL;
+
+  if (!the_object)
+    the_object = g_object_new (OOBS_TYPE_USERS_CONFIG,
+                               "remote-object", USERS_CONFIG_REMOTE_OBJECT,
+                               NULL);
+
+  return the_object;
 }
 
 /**
@@ -623,6 +452,114 @@ oobs_users_config_get_users (OobsUsersConfig *config)
   return priv->users_list;
 }
 
+/**
+ * oobs_users_config_add_user:
+ * @config: An #OobsUsersConfig.
+ * @user: An #OobsUser.
+ *
+ * Add an user to the configuration, immediately committing changes to the system.
+ * On success, @user will be appended to the users list.
+ *
+ * Return value: an #OobsResult enum with the error code.
+ **/
+OobsResult
+oobs_users_config_add_user (OobsUsersConfig *config, OobsUser *user)
+{
+  OobsUsersConfigPrivate *priv;
+  OobsListIter list_iter;
+  OobsResult result;
+
+  g_return_val_if_fail (config != NULL, OOBS_RESULT_MALFORMED_DATA);
+  g_return_val_if_fail (user != NULL, OOBS_RESULT_MALFORMED_DATA);
+  g_return_val_if_fail (OOBS_IS_USERS_CONFIG (config), OOBS_RESULT_MALFORMED_DATA);
+  g_return_val_if_fail (OOBS_IS_USER (user), OOBS_RESULT_MALFORMED_DATA);
+
+  result = oobs_object_add (OOBS_OBJECT (user));
+
+  if (result != OOBS_RESULT_OK)
+    return result;
+
+  priv = config->_priv;
+
+  oobs_list_append (priv->users_list, &list_iter);
+  oobs_list_set (priv->users_list, &list_iter, G_OBJECT (user));
+
+  return OOBS_RESULT_OK;
+}
+
+/**
+ * oobs_users_config_delete_user:
+ * @config: An #OobsUsersConfig.
+ * @user: An #OobsUser.
+ *
+ * Delete an user from the configuration, immediately committing changes to the system.
+ * On success, @user will be removed from the users list.
+ *
+ * Return value: an #OobsResult enum with the error code.
+ **/
+OobsResult
+oobs_users_config_delete_user (OobsUsersConfig *config, OobsUser *user)
+{
+  OobsUsersConfigPrivate *priv;
+  OobsUser *list_user;
+  OobsListIter list_iter;
+  gboolean valid;
+  OobsResult result;
+  OobsList *groups_list;
+  OobsGroup *group;
+
+  g_return_val_if_fail (config != NULL, OOBS_RESULT_MALFORMED_DATA);
+  g_return_val_if_fail (user != NULL, OOBS_RESULT_MALFORMED_DATA);
+  g_return_val_if_fail (OOBS_IS_USERS_CONFIG (config), OOBS_RESULT_MALFORMED_DATA);
+  g_return_val_if_fail (OOBS_IS_USER (user), OOBS_RESULT_MALFORMED_DATA);
+
+  /* Try to commit changes */
+  result = oobs_object_delete (OOBS_OBJECT (user));
+
+  if (result != OOBS_RESULT_OK)
+    return result;
+
+
+  priv = config->_priv;
+
+  /* Remove user from all groups, to avoid committing to /etc/group
+   * the name of a non-existent user */
+  groups_list = oobs_groups_config_get_groups (OOBS_GROUPS_CONFIG (oobs_groups_config_get ()));
+
+  valid = oobs_list_get_iter_first (groups_list, &list_iter);
+
+  while (valid) {
+    group = OOBS_GROUP (oobs_list_get (groups_list, &list_iter));
+
+    oobs_group_remove_user (group, user);
+
+    g_object_unref (group);
+
+    valid = oobs_list_iter_next (priv->users_list, &list_iter);
+  }
+
+
+  /* Then remove user from the list */
+  valid = oobs_list_get_iter_first (priv->users_list, &list_iter);
+
+  while (valid) {
+    list_user = OOBS_USER (oobs_list_get (priv->users_list, &list_iter));
+
+    if (list_user == user) {
+      g_object_unref (list_user);
+      break;
+    }
+
+    g_object_unref (list_user);
+
+    valid = oobs_list_iter_next (priv->users_list, &list_iter);
+  }
+
+  oobs_list_remove (priv->users_list, &list_iter);
+
+  return OOBS_RESULT_OK;
+}
+
 /**
  * oobs_users_config_get_minimum_users_uid:
  * @config: An #OobsUsersConfig.
@@ -760,7 +697,7 @@ oobs_users_config_get_default_home_dir (OobsUsersConfig *config)
 
 /**
  * oobs_users_config_set_default_home_dir:
- * @config: An #OobsIfacesConfig.
+ * @config: An #OobsUsersConfig.
  * @home_dir: new default home directory prefix.
  * 
  * Sets a new home directory prefix used for newly created users, replacing the old one.
@@ -817,3 +754,235 @@ oobs_users_config_get_available_shells (OobsUsersConfig *config)
 
   return priv->shells;
 }
+
+/**
+ * oobs_users_config_get_encrypted_home_support:
+ * @config: An #OobsUsersConfig
+ *
+ * Returns whether encrypted home directories are supported by the platform
+ * (e.g. using eCryptfs).
+ *
+ * Return Value: %TRUE if encrypted home dirs are supported, %FALSE otherwise.
+ **/
+gboolean
+oobs_users_config_get_encrypted_home_support (OobsUsersConfig *config)
+{
+  OobsUsersConfigPrivate *priv;
+
+  g_return_val_if_fail (config != NULL, FALSE);
+  g_return_val_if_fail (OOBS_IS_USERS_CONFIG (config), FALSE);
+
+  priv = config->_priv;
+
+  return priv->encrypted_home;
+}
+
+/*
+ * Convenience functions to make working with the users list easier.
+ */
+
+/**
+ * oobs_users_config_get_from_login:
+ * @config: An #OobsUsersConfig.
+ * @login: the login name of the wanted user.
+ *
+ * Gets the (first) user whose login is @login. This is a convenience function
+ * to avoid walking manually over the users list.
+ *
+ * Return value: an #OobsUser corresponding to the passed login,
+ * or %NULL if no such user exists. Don't forget to unref user when you're done.
+ **/
+OobsUser *
+oobs_users_config_get_from_login (OobsUsersConfig *config, const gchar *login)
+{
+  OobsUsersConfigPrivate *priv;
+  OobsUser *user;
+  OobsListIter iter;
+  gboolean valid;
+  const gchar *user_login;
+
+  g_return_val_if_fail (config != NULL, NULL);
+  g_return_val_if_fail (OOBS_IS_USERS_CONFIG (config), NULL);
+  g_return_val_if_fail (login != NULL, NULL);
+
+  priv = config->_priv;
+
+  valid = oobs_list_get_iter_first (priv->users_list, &iter);
+
+  while (valid) {
+    user = OOBS_USER (oobs_list_get (priv->users_list, &iter));
+    user_login = oobs_user_get_login_name (user);
+
+    if (user_login && strcmp (login, user_login) == 0)
+      return user;
+
+    /* only the returned user is not unreferenced here */
+    g_object_unref (user);
+
+    valid = oobs_list_iter_next (priv->users_list, &iter);
+  }
+
+  return NULL;
+}
+
+/**
+ * oobs_users_config_get_from_uid:
+ * @config: An #OobsUsersConfig.
+ * @uid: the UID of the wanted user.
+ *
+ * Gets the (first) user whose UID is @uid. This is a convenience function
+ * to avoid walking manually over the users list.
+ *
+ * Return value: an #OobsUser corresponding to the passed UID,
+ * or %NULL if no such user exists. Don't forget to unref user when you're done.
+ **/
+OobsUser *
+oobs_users_config_get_from_uid (OobsUsersConfig *config, uid_t uid)
+{
+  OobsUsersConfigPrivate *priv;
+  OobsUser *user;
+  OobsListIter iter;
+  gboolean valid;
+  uid_t user_uid;
+
+  g_return_val_if_fail (config != NULL, NULL);
+  g_return_val_if_fail (OOBS_IS_USERS_CONFIG (config), NULL);
+
+  priv = config->_priv;
+
+  valid = oobs_list_get_iter_first (priv->users_list, &iter);
+
+  while (valid) {
+    user = OOBS_USER (oobs_list_get (priv->users_list, &iter));
+    user_uid = oobs_user_get_uid (user);
+
+    if (user_uid == uid)
+      return user;
+
+    /* only the returned user is not unreferenced here */
+    g_object_unref (user);
+
+    valid = oobs_list_iter_next (priv->users_list, &iter);
+  }
+
+  return NULL;
+}
+
+/**
+ * oobs_users_config_is_login_used:
+ * @config: An #OobsUsersConfig.
+ * @login: the login name to check.
+ *
+ * Check whether @login is already used by an existing user or not. This is
+ * a convenience function to avoid walking manually over the users list.
+ *
+ * Return value: %TRUE if an user named @login already exists, %FALSE otherwise.
+ **/
+gboolean
+oobs_users_config_is_login_used (OobsUsersConfig *config, const gchar *login)
+{
+  OobsUser *user;
+  gboolean login_used;
+
+  user = oobs_users_config_get_from_login (config, login);
+  login_used = (user != NULL);
+
+  if (user)
+    g_object_unref (user);
+
+  return login_used;
+}
+
+/**
+ * oobs_users_config_is_uid_used:
+ * @config: An #OobsUsersConfig.
+ * @uid: the UID to check.
+ *
+ * Check whether @uid is already used by an existing user or not. This is
+ * a convenience function to avoid walking manually over the users list.
+ *
+ * Return value: %TRUE if an user with such an UID already exists, %FALSE otherwise.
+ **/
+gboolean
+oobs_users_config_is_uid_used (OobsUsersConfig *config, uid_t uid)
+{
+  OobsUser *user;
+  gboolean uid_used;
+
+  user = oobs_users_config_get_from_uid (config, uid);
+  uid_used = (user != NULL);
+
+  if (user)
+    g_object_unref (user);
+
+  return uid_used;
+}
+
+/**
+ * oobs_users_config_find_free_uid:
+ * @config: An #OobsUsersConfig.
+ * @uid_min: the minimum wanted UID.
+ * @uid_max: the maximum wanted UID.
+ *
+ * Finds an UID that is not used by any user in the list. The returned UID is
+ * the highest used UID in the range plus one if @uid_max is not used.
+ * Else, the first free UID in the range is returned.
+ *
+ * If both @uid_min and @uid_max are equal to 0, the default range is used.
+ *
+ * Return value: a free UID in the requested range,
+ * or @uid_max to indicate wrong use or failure to find a free UID.
+ **/
+uid_t
+oobs_users_config_find_free_uid (OobsUsersConfig *config, uid_t uid_min, uid_t uid_max)
+{
+  OobsUsersConfigPrivate *priv;
+  OobsList *list;
+  OobsListIter list_iter;
+  OobsUser *user;
+  gboolean valid;
+  uid_t new_uid, user_uid;
+
+  g_return_val_if_fail (config != NULL, uid_max);
+  g_return_val_if_fail (OOBS_IS_USERS_CONFIG (config), uid_max);
+  g_return_val_if_fail (uid_min <= uid_max, uid_max);
+
+  priv = config->_priv;
+
+  if (uid_min == 0 && uid_max == 0) {
+    uid_min = priv->minimum_uid;
+    uid_max = priv->maximum_uid;
+  }
+
+  new_uid = uid_min - 1;
+
+  list = oobs_users_config_get_users (config);
+  valid = oobs_list_get_iter_first (list, &list_iter);
+
+  /* Find the highest used UID in the range */
+  while (valid) {
+    user = OOBS_USER (oobs_list_get (list, &list_iter));
+    user_uid = oobs_user_get_uid (user);
+    g_object_unref (user);
+
+    if (user_uid < uid_max && user_uid >= uid_min && user_uid > new_uid)
+      new_uid = user_uid;
+
+    valid = oobs_list_iter_next (list, &list_iter);
+  }
+
+  new_uid++;
+
+  if (!oobs_users_config_is_uid_used (config, new_uid))
+    return new_uid;
+
+
+  /* If the fast method failed, iterate over the whole range */
+  new_uid = uid_min;
+  while (oobs_users_config_is_uid_used (config, new_uid) && new_uid < uid_max)
+    new_uid++;
+
+  /* In the extreme case where no UID is free in the range,
+   * we return the uid_max, which is the best we can do */
+  return new_uid;
+}
